package com.faraphel.tasks_valider.connectivity.task.session

import com.faraphel.tasks_valider.database.entities.PersonEntity
import fi.iki.elonen.NanoHTTPD
import java.util.*


/**
 * The manager for the session system
 */
class TaskSessionManager(///< the admin person entity
    private val adminPersonEntity: PersonEntity
) {
    private val sessions = mutableMapOf<String, TaskSession>()  ///< sessions specific data

    private val adminPersonData = this.newSessionData(this.adminPersonEntity)  ///< the session of the admin
    val adminSessionToken = this.adminPersonData.first  ///< the session token of the admin
    val adminSession = this.adminPersonData.second  ///< the session of the admin

    /**
     * Get the admin person entity
     * @return the admin person entity
     */
    fun getAdminPersonEntity(): PersonEntity {
        return this.adminPersonEntity
    }

    /**
     * Create a new session
     * @param person the person for the session (optional)
     * @return a new session identifier and the corresponding session
     */
    fun newSessionData(person: PersonEntity): Pair<String, TaskSession> {
        // create a new session token that is a secret random string
        val sessionToken: String = UUID.randomUUID().toString()
        // create a new session
        val session = TaskSession(person)
        // store the session
        this.sessions[sessionToken] = session
        // return the session data
        return Pair(sessionToken, session)
    }

    /**
     * Get data from a http session
     * @param httpSession the HTTP session
     * @return the session data
     */
    fun getSessionData(httpSession: NanoHTTPD.IHTTPSession): Pair<String, TaskSession>? {
        // get the session token from the cookies
        val sessionToken = httpSession.cookies.read("sessionToken") ?: return null
        // get the session data
        val sessionData = this.getSessionData(sessionToken)
        // return the session data
        return sessionData
    }

    /**
     * Get data from a session identifier
     * @param sessionToken the identifier of the session
     * @return the session data
     */
    fun getSessionData(sessionToken: String): Pair<String, TaskSession>? {
        // get the session data
        val session = this.sessions[sessionToken]
            ?: return null
        // return the session data
        return Pair(sessionToken, session)
    }

    /**
     * Update the session data
     * @param sessionToken the identifier of the session
     * @param session the new session data
     */
    fun updateSessionData(sessionToken: String, session: TaskSession) {
        // update the session
        this.sessions[sessionToken] = session
    }

    /**
     * Delete a session
     * @param sessionToken the identifier of the session
     */
    fun deleteSessionData(sessionToken: String): Pair<String, TaskSession>? {
        // remove the session
        val session = this.sessions.remove(sessionToken) ?:
            return null
        // return the session data
        return Pair(sessionToken, session)
    }

    /**
     * Get data from a http session. If it does not exist, create it.
     * @param httpSession the HTTP session
     */
    fun getOrCreateSessionData(httpSession: NanoHTTPD.IHTTPSession, person: PersonEntity): Pair<String, TaskSession> {
        // try to get the session directly
        val sessionData = this.getSessionData(httpSession)
            ?: this.newSessionData(person)
        // return the session data
        return sessionData
    }

    /**
     * Insert new cookies for a session in a response
     * @param response the response to inject cookies into
     * @param cookies the cookie handler
     */
    fun responseSetSessionData(
        response: NanoHTTPD.Response,
        cookies: NanoHTTPD.CookieHandler,
        sessionToken: String
    ): NanoHTTPD.Response {
        // update the cookie of the user
        cookies.set(NanoHTTPD.Cookie("sessionToken", sessionToken))
        // load them in the response
        cookies.unloadQueue(response)

        return response
    }
}