package com.faraphel.tasks_valider.database.api.server.entities.base

import com.faraphel.tasks_valider.database.dao.base.BaseTaskDao
import com.faraphel.tasks_valider.database.entities.SessionEntity
import com.faraphel.tasks_valider.utils.getBody
import com.faraphel.tasks_valider.utils.parser
import fi.iki.elonen.NanoHTTPD


abstract class BaseTaskDatabaseApi<Entity> (
    private val dao: BaseTaskDao<Entity>,
    private val session: SessionEntity,
    private val entityType: Class<Entity>,
) : BaseDatabaseApi {
    /**
     * Handle an HTTP HEAD request.
     * Indicate if an object exist in the database.
     * @param httpSession the current http session to handle.
     * @return a response indicating if the object does exist.
     */
    override fun head(httpSession: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        // get the content of the request
        val data = httpSession.getBody()
        // parse the object
        val obj = parser.fromJson(data, entityType)
        // check if the object is in the object accessible from the session
        val exists = this.dao.getAllBySession(session.id).contains(obj)

        // return the response
        return NanoHTTPD.newFixedLengthResponse(
            if (exists) NanoHTTPD.Response.Status.OK else NanoHTTPD.Response.Status.NOT_FOUND,
            "text/plain",
            if (exists) "Exists" else "Not found"
        )
    }

    /**
     * Handle an HTTP GET request.
     * Indicate the content of a table in the database.
     * @param httpSession the current http session to handle.
     * @return a response indicating the content of all the objects.
     */
    override fun get(httpSession: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        // return the content of all the objects in the database about this session
        return NanoHTTPD.newFixedLengthResponse(
            NanoHTTPD.Response.Status.OK,
            "application/json",
            parser.toJson(this.dao.getAllBySession(session.id))
        )
    }

    /**
     * Handle an HTTP POST request.
     * Create a new object in the database.
     * @param httpSession the current http session to handle.
     * @return the id of the newly created object
     */
    override fun post(httpSession: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        // get the content of the request
        val data = httpSession.getBody()
        // parse the object
        val obj = parser.fromJson(data, entityType)
        // insert it into the database
        val id = this.dao.insert(obj)

        // return the id of the object created
        return NanoHTTPD.newFixedLengthResponse(
            NanoHTTPD.Response.Status.CREATED,
            "text/plain",
            id.toString()
        )
    }

    /**
     * Handle an HTTP DELETE request.
     * Delete an item from the database.
     * @param httpSession the current http session to handle.
     * @return the number of object deleted.
     */
    override fun delete(httpSession: NanoHTTPD.IHTTPSession): NanoHTTPD.Response {
        // get the content of the request
        val data = httpSession.getBody()
        // parse the object
        val obj = parser.fromJson(data, entityType)
        // delete all the instance of this object in the database
        val count = this.dao.delete(obj)

        // return the number of corresponding element deleted
        return NanoHTTPD.newFixedLengthResponse(
            if (count > 0) NanoHTTPD.Response.Status.OK else NanoHTTPD.Response.Status.NOT_FOUND,
            "text/plain",
            count.toString()
        )
    }
}