package com.faraphel.tasks_valider.ui.screen.task

import android.app.Activity
import android.util.Log
import android.widget.Toast
import androidx.compose.foundation.layout.*
import androidx.compose.material3.Button
import androidx.compose.material3.Text
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.navigation.NavController
import androidx.navigation.compose.NavHost
import androidx.navigation.compose.composable
import androidx.navigation.compose.rememberNavController
import com.faraphel.tasks_valider.connectivity.task.TaskClient
import com.faraphel.tasks_valider.connectivity.task.session.TaskRole
import com.faraphel.tasks_valider.database.entities.PersonEntity
import com.faraphel.tasks_valider.utils.parser
import java.io.File


@Composable
fun TaskSessionController(
    activity: Activity,
    client: TaskClient,
    user: PersonEntity,
) {
    val controller = rememberNavController()

    NavHost(
        navController = controller,
        startDestination = "main"
    ) {
        composable("main") {
            TaskSessionScreen(controller, activity, client, user)
        }
        composable("quick_validation") {
            QuickValidationScreen(controller, activity, client, user)
        }
    }
}


/**
 * This screen represent a session
 * @param activity the android activity
 * @param client an HTTP client that can communicate with the server
 */
@Composable
fun TaskSessionScreen(
    controller: NavController,
    activity: Activity,
    client: TaskClient,
    user: PersonEntity,
) {
    val students = remember { mutableStateOf<List<PersonEntity>?>(null) }
    val selectedStudent = remember { mutableStateOf<PersonEntity?>(null) }

    // if the groups are not yet defined, refresh the list
    if (students.value == null)
        return LaunchedEffect(true) {
            Thread { refreshStudents(activity, client, students) }.start()
        }

    if (selectedStudent.value != null)
        return TaskStudentScreen(
            activity,
            client,
            user,
            selectedStudent.value!!
        )

    Column(
        modifier = Modifier.fillMaxSize(),
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        // title
        Text(
            text = "Session",
            fontSize = 32.sp
        )

        // separator
        Spacer(modifier = Modifier.height(24.dp))

        // if the groups have already been defined, display them
        for (student in students.value!!) {
            Button(
                modifier = Modifier.fillMaxWidth().padding(vertical = 2.dp, horizontal = 16.dp),
                onClick = { selectedStudent.value = student },
            ) {
                Text(text = student.fullName())
            }
        }

        // separator
        Spacer(modifier = Modifier.weight(1f))

        // buttons
        Row {
            // quick validation
            Button(onClick = { controller.navigate("quick_validation") }) {
                Text("Quick Validation")
            }
            Button(onClick = { Thread { exportToFile(activity, client) }.start() }) {
                Text("Export")
            }
        }
    }
}


fun refreshStudents(
    activity: Activity,
    client: TaskClient,
    students: MutableState<List<PersonEntity>?>
) {
    try {
        // try to get all the persons in that session
        students.value = client.entities.person.getAll()
    } catch (exception: Exception) {
        // in case of error, show a message
        return activity.runOnUiThread {
            Log.e("students", "$exception")
            Toast.makeText(activity, "Could not retrieve students.\n\n$exception", Toast.LENGTH_LONG).show()
        }
    }
}


fun exportToFile(
    activity: Activity,
    client: TaskClient,
) {
    // get all the values to export
    val allPersons = client.entities.person.getAll()
    val allStudents = allPersons.filter { student -> student.role == TaskRole.STUDENT }
    val allRelationsStudentSessionSubject = client.entities.relationPersonSessionSubject.getAll()
    val allSubjects = client.entities.subject.getAll()
    val allTasks = client.entities.task.getAll()
    val allValidations = client.entities.validation.getAll()

    // for each student

    val data = allStudents.map { student ->
        // fetch their subject
        val relationStudentSessionObject = allRelationsStudentSessionSubject.first { relation ->
            relation.studentId == student.id
        }
        val subject = allSubjects.first { subject -> subject.id == relationStudentSessionObject.subjectId }
        // get the tasks of this person
        val tasks = allTasks.filter { task -> task.subjectId == subject.id }

        // for all the tasks
        val studentTasksData = tasks.map { task ->
            // if a validation match
            val isValidated = allValidations.any { validation -> validation.taskId == task.id }
            // return the pair of the task title and its validation
            mapOf(
                "name" to task.title,
                "validated" to isValidated,
            )
        }

        // return the name of the student associated to his data
        mapOf(
            "name" to student.fullName(),
            "subject" to subject.name,
            "tasks" to studentTasksData,
        )
    }

    val dataJson = parser.toJson(data)
    Log.i("export", dataJson)

    // write all the data in a json file
    val file = File(activity.filesDir, "test.json")
    file.writeText(dataJson)

    // TODO(Faraphel): prompt to open the file ?
}